use libloading::{Library, Symbol};
use std::process::Command;

use ittapi::{Domain, StringHandle, Task};
#[link(name = "mylib")]
extern "C" {
    fn matmul(len_i: i32, len_j: i32, len_k: i32, A: *const f32, B: *const f32, C: *mut f32);
}

fn main() {
    let len_i = 1000;
    let len_j = 1000;
    let len_k = 1000;
    let a: Vec<f32> = (0..(len_i * len_k)).map(|x| x as f32).collect();
    let b: Vec<f32> = (0..(len_k * len_j)).map(|x| 0.5 + x as f32).collect();
    let mut c: Vec<f32> = std::iter::repeat(0.0).take(len_i * len_j).collect();
    let domain = Domain::new("domain");
    let name = StringHandle::new("taskMatmulStatic");
    unsafe {
        let task = Task::begin(&domain, name);
        matmul(
            len_i as i32,
            len_j as i32,
            len_k as i32,
            a.as_ptr(),
            b.as_ptr(),
            c.as_mut_ptr(),
        );
        task.end();
    }

    println!("c = {:?}", &c[0..10]);
    c.iter_mut().for_each(|x| *x = 0.0);

    let name2 = StringHandle::new("taskMatmulDynamic");
    unsafe {
        let compile_ok = Command::new("gcc")
            .args(["-O3", "-shared", "-o", "mylib.so", "src/mylib.c"])
            .status()
            .unwrap()
            .success();
        assert!(compile_ok);
        let lib = Library::new("./mylib.so").unwrap();
        let symbol: Symbol<extern "C" fn(i32, i32, i32, *const f32, *const f32, *mut f32)> =
            lib.get("matmul".as_bytes()).unwrap();
        let task = Task::begin(&domain, name2);
        symbol(
            len_i as i32,
            len_j as i32,
            len_k as i32,
            a.as_ptr(),
            b.as_ptr(),
            c.as_mut_ptr(),
        );
        task.end();
        println!("c = {:?}", &c[0..10]);
    }
}
